package com.jpattern.gwt.client.presenter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.google.gwt.user.client.ui.HasWidgets;
import com.jpattern.gwt.client.navigationevent.INavigationEvent;
import com.jpattern.gwt.client.navigationevent.INavigationEventCallback;
import com.jpattern.gwt.client.navigationevent.INavigationEventData;
import com.jpattern.gwt.client.navigationevent.INavigationEventWrapper;
import com.jpattern.gwt.client.navigationevent.NavigationEventData;
import com.jpattern.gwt.client.navigationevent.NavigationEventWrapper;
import com.jpattern.gwt.client.navigationevent.NullNavigationEventData;
import com.jpattern.gwt.client.view.IShowViewStrategy;
import com.jpattern.gwt.client.view.IView;
import com.jpattern.shared.result.IErrorMessage;

/**
 * 
 * @author Francesco Cina'
 *
 * 14 Apr 2011
 */

public abstract class APresenter<T extends IView> implements IPresenter {
	
	private final IShowViewStrategy<T> showViewStrategy;
	private IPresenter parentPresenter;
	private final INavigationEvent currentNavigationEvent;
	private final Map<String, INavigationEventData> registeredNavigationEvents = new HashMap<String, INavigationEventData>();
	private final Map<String, Map<String,String>> initialNavigationEvents = new HashMap<String, Map<String,String>>();
	private final List<String> initialNavigationEventsToIgnore = new ArrayList<String>();
	private boolean initialized = false;

	public APresenter(IShowViewStrategy<T> showViewStrategy, INavigationEvent navigationEvent) {
		this.showViewStrategy = showViewStrategy;
		this.currentNavigationEvent = navigationEvent;
		showViewStrategy.getView().visit(this);
	}
	
	@Override
	public final void onEventError(List<IErrorMessage> errorMessages) {
		showViewStrategy.getView().getErrorArea().addErrorMessages(errorMessages);
	}
	
	@Override
	public final void onEventStart() {
		showViewStrategy.getView().getErrorArea().clear();
	}
	
	@Override
	public final T getView() {
		return showViewStrategy.getView();
	}
	
	/**
	 * This is the first step performed to a complete visualization of the associated IView.
	 * This method is executed only once when the Presenter is created.
	 * This method is called just after the IShowViewStrategy.onLoadStart() method and before the
	 * IShowViewStrategy.onLoadCompleted() method.
	 * Here the asynchronous actions should be performed.
	 * It's VERY important to call the render() method when all the actions are performed otherwise
	 * the preRender() phase will never be called and the IView should not be completely rendered. 
	 */
	protected abstract void init();
	
	/**
	 * This is the second step performed to a complete visualization of the associated IView.
	 * this method is performed every time the render() method is called on the Presenter.
	 * This is performed just before the IShowViewStrategy.onLoadStart() method.
	 * This method is called before the display of the IView.
	 * DO NOT call render() inside this method! This generates an infinite loop!
	 */
	protected abstract void preRender();
	
	/**
	 * This method must be called when all the necessary operations to load necessary data are ended.
	 * This causes the final rendering of the IView. 
	 */
	@Override
	public final void render() {
		preRender();
		showViewStrategy.onLoadCompleted();
		initialized = true;
	}

	@Override
	public final void setParent(IPresenter parentPresenter) {
		this.parentPresenter = parentPresenter;
		showViewStrategy.setParent(getParentPresenter().getNavigationEventData(getNavigationEvent().getName()).getEventTarget());
	}
	
	@Override
	public final void initPresenter() {
		if (!initialized) {
			onEventStart();
			showViewStrategy.onLoadStart();
			init();
			launchInitialEvents();
		} else {
			render();
		}
	}
	
	@Override
	public final void hierarchy(List<String> hierarchyResult) {
		getParentPresenter().hierarchy(hierarchyResult);
		hierarchyResult.add(getName());
	}
	
	@Override
	public final void registerNavigationEvent(INavigationEvent navigationEvent, HasWidgets eventTarget, INavigationEventCallback navigationEventCallback) {
		registerNavigationEvent(new NavigationEventWrapper(navigationEvent), eventTarget, navigationEventCallback);
	}
	
	@Override
	public void registerNavigationEvent(INavigationEventWrapper navigationEventWrapper, HasWidgets eventTarget,	INavigationEventCallback navigationEventCallback) {
		registeredNavigationEvents.put(navigationEventWrapper.getName(), new NavigationEventData(navigationEventWrapper, eventTarget, navigationEventCallback));
	}
	
	@Override
	public final void launchNavigationEvent(String navigationEventName, boolean registerHistory) {
		launchNavigationEvent(navigationEventName, registerHistory, new HashMap<String, String>());
	}
	
	@Override
	public final void launchNavigationEvent(String navigationEventName, boolean registerHistory, Map<String, String> queryStringValues) {
		if (!localLaunchNavigationEvent(navigationEventName, registerHistory, queryStringValues)) {
			getParentPresenter().launchNavigationEvent(navigationEventName, registerHistory);
		} 
	}
	
	private boolean localLaunchNavigationEvent(String navigationEventName, boolean registerHistory, Map<String, String> queryStringValues) {
		if (registeredNavigationEvents.containsKey(navigationEventName)) {
			INavigationEventData eventData = registeredNavigationEvents.get(navigationEventName);
			eventData.getNavigationEventWrapper().notifyNavigationEvent(this, queryStringValues, registerHistory);
			eventData.getNavigationEventCallback().callback();
			return true;
		} 
		return false;
	}
	
	@Override
	public final INavigationEventData getNavigationEventData(String navigationEventName) {
		if (registeredNavigationEvents.containsKey(navigationEventName)) {
			return registeredNavigationEvents.get(navigationEventName);
		}
		return new NullNavigationEventData();
	}

	
	protected final IPresenter getParentPresenter() {
		if (parentPresenter==null) {
			parentPresenter = new NullPresenter();
		}
		return parentPresenter;
	}


	@Override
	public final INavigationEvent getNavigationEvent() {
		return currentNavigationEvent;
	}

	@Override
	public final boolean isInitialized() {
		return initialized;
	}

	@Override
	public final void addInitialNavigationEvent(String navigationEventName) {
		addInitialNavigationEvent(navigationEventName, new HashMap<String, String>());
	}
	
	@Override
	public final void addInitialNavigationEvent(String navigationEventName, Map<String, String> queryStringValues) {
		INavigationEventData newEventData = getNavigationEventData(navigationEventName);
		HasWidgets eventTarget = newEventData.getEventTarget();
		boolean targetInUse = false;
		for (Entry<String, Map<String, String>> entry : initialNavigationEvents.entrySet()) {
			if (getNavigationEventData(entry.getKey()).getEventTarget().equals(eventTarget)) {
				targetInUse = true;
				break;
			}
		}
		if (!targetInUse) {
			initialNavigationEvents.put(navigationEventName, queryStringValues);
		}
	}
	
	@Override
	public void ignoreInitialEventWithSameTargetOf(String navigationEventName) {
		INavigationEventData newEventData = getNavigationEventData(navigationEventName);
		HasWidgets eventTarget = newEventData.getEventTarget();
		for (Entry<String, Map<String, String>> entry : initialNavigationEvents.entrySet()) {
			if (getNavigationEventData(entry.getKey()).getEventTarget().equals(eventTarget)) {
				initialNavigationEventsToIgnore.add(entry.getKey());
			}
		}
	}
	
	private final void launchInitialEvents() {
		for (Entry<String, Map<String, String>> entry : initialNavigationEvents.entrySet()) {
			if ( !initialNavigationEventsToIgnore.contains(entry.getKey()) ) {
				localLaunchNavigationEvent(entry.getKey(), false, entry.getValue());
			}
		}
		initialNavigationEventsToIgnore.clear();
	}

	@Override
	public String getName() {
		return getNavigationEvent().getName();
	}
	
}
